import {select, 
        csv, 
        scaleLinear,
        max, 
        scalePoint,
      	axisLeft,
       	axisBottom,
        format} from 'd3';

const svg = select('svg');

const height =+svg.attr('height');
const width =+svg.attr('width');


const render = data => {
  //SPECIFIC TO DATA SET, CHANGE XVAL AND YVAL 
  const xValue = d => d.population;
  const yValue = d => d.country;
  const margin = { top: 50, right: 60, bottom: 70, left: 95 };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;

  //START - GENERALIZED CODE TO DATA SET
  const xScale = scaleLinear() //create instance of scaleLinear
  	.domain([0, max(data, xValue)]) //computes max from population
  	.range([0, innerWidth])
  	.nice(); //makes domain end on a tick (1.6B in this example) 
  
  const yScale = scalePoint()
  	.domain(data.map(yValue))
  	.range([0, innerHeight])
  	.padding(0.8); //space b/w bars
  
  const xAxisTickFormat = number => 
  		format('.3s')(number)  // 3 sig figs
  			.replace('G', 'B');
  
  const yAxis = axisLeft(yScale)
  							.tickSize(-innerWidth);
  const xAxis = axisBottom(xScale)
  							.tickFormat(xAxisTickFormat)
  							.tickSize(-innerHeight); //makes ticks extend all the way up
  
  const group = svg.append('g')
  	.attr('transform', `translate(${margin.left},${margin.top})`);
  
  group.append('g')
    .call(yAxis)
  	.selectAll('.domain') //selector strings like css and html class selectors
    	.remove(); 
  
  const xAxisGroup = group.append('g')
    .call(xAxis)
  	.attr('transform', `translate(0,${innerHeight})`);
  
  xAxisGroup.select('.domain').remove();
  
  xAxisGroup.append('text')
  	.attr('class', 'axis-label')
  	.attr('x', innerWidth/2)
  	.attr('y', 40)
    .attr('fill', 'black')
  	.text('Population')
  
	group.selectAll('circle').data(data) //data join elements of data to DOM rect
  	.enter().append('circle') 
  		.attr('cy', d => yScale(yValue(d)))
  		.attr('cx', d => xScale(xValue(d))) //pass in 1 row of data, specifically population
  		.attr('r', 8); // bandwidth = computed width of single bar
  
  group.append('text')
  	.attr('class', 'graph-title')
  	.attr('y', -10)
  	.text('Top 10 Most Populous Countries');
  //END - GENERALIZED CODE
};


/*
returns a promise to be loaded -> then
xml http request, load csv string, parses csv string -> array of objs
key of each obj = col, val = val
*/
csv('country_data.csv').then(data => {
  data.forEach(d =>{
    //unary operator parses string -> values, data in thousands so *1000
    d.population = +d.population * 1000; 
  });
  // console.log(data);
  render(data);
});  